/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/***********************************************************
Base class that start and end of the search event.

***********************************************************/

INCLUDE("bookmarkToolsJSHelper.js");

function BaseSearch(){}

BaseSearch.prototype = {
	startSearch: function(searchString, searchParam, previousResult, listener) {
		var defaultIndex = 0;
		var matchCount = 0;
		var searchResult = 0;
		this.searchSite = "";

		var browserWindow = NativeComponents.nsIBrowserWindow();
		this.searchString = browserWindow.gURLBar.value;

		this.searchPhrase = this.searchString;
		this.searchPage = 0;

		this._results = []
		this._listener = listener;

		this.init();
		this._execute();
	},

	stopSearch: function(){

	},

	_execute: function(event){
		this._results = [];
		if(this.execute()){
			this.matchCount = this._results.length;
			this.searchResult = Ci.nsIAutoCompleteResult.RESULT_SUCCESS;
			this._listener.onSearchResult(this, this);
		} else{
			this.matchCount = 0;
			this.searchResult = Ci.nsIAutoCompleteResult.RESULT_FAILURE;
			this._results = [];
			this._listener.onSearchResult(this, this);
		}
	},

	getCommentAt: function(index){
		if(!this._results || index >= this._results.length){
			return null;
		}
		return this._results[index].comment;
	},

	getImageAt: function(index){
		if(!this._results || index >= this._results.length){
			return null;
		}
		return this._results[index].image;
	},

	getStyleAt: function(index){
		if(!this._results || index >= this._results.length){
			return null;
		}
		return this._results[index].style;
	},

	getValueAt: function(index){
		if(!this._results || index >= this._results.length){
			return null;
		}
		return this._results[index].value;
	},

	removeValueAt: function(index, removeFromDb){ },

	QueryInterface: function(iid) { 
		if (!iid.equals(Ci.nsIAutoCompleteSearch) &&
		!iid.equals(Ci.nsIAutoCompleteObserver) &&
		!iid.equals(Ci.nsISupports)) {   
			throw Cr.NS_ERROR_NO_INTERFACE; 
        }
        return this;
    }
};

/***********************************************************
Component extends base class and executes search events.

***********************************************************/

IndexSearch = function(){ };
IndexSearch.prototype = new BaseSearch();

IndexSearch.prototype.init = function(){
	var instReader = Components.classes["@bookmarktools.mozdev.org/search/indexReader;1"].createInstance();
	this.indexReader = instReader.QueryInterface(Components.interfaces.ICLReader);

	this.indexReader.setPath(BT.IO.getSearchDir().path);
}

IndexSearch.prototype.search = function(size){
	if(this.indexReader.exists()){
	
		this.searchString = StringUtils.trim(this.searchString);
	
		if(this.searchString && this.searchString.length > 2){
			var array = NativeComponents.nsIMutableArray();
			this.indexReader.search(this.searchString, array, 0, size);

			return array;
		}
	}
}

IndexSearch.prototype.execute = function(){
	try {
	
		var size = Prefs.getInt("addressbar");
		if(size > 0){
			var array = this.search(size);
			var e = array.enumerate();
		
			var bmsvc = Components.classes["@mozilla.org/browser/nav-bookmarks-service;1"]
			.getService(Components.interfaces.nsINavBookmarksService);

			var ioService = Components.classes["@mozilla.org/network/io-service;1"]
			.getService(Components.interfaces.nsIIOService);
			
			var faviconService = Components.classes["@mozilla.org/browser/favicon-service;1"]
			.getService(Components.interfaces.nsIFaviconService);
								
			var truncateSize = 100;
			var truncateShortCut = " [..]";
			
			while (e.hasMoreElements()) {
				var resultNode = e.getNext().QueryInterface(Components.interfaces.IResultNode);
				var uri = ioService.newURI(resultNode.primaryKey, null, null);
				
				if(bmsvc.isBookmarked(uri)){
				
					var itemIDs = bmsvc.getBookmarkIdsForURI(uri, {});
					var iconUri = "chrome://browser/skin/Bookmarks-folder.png";
			
					try {
						iconUri = faviconService.getFaviconForPage(uri).spec;
					} catch(e) {
						iconUri = "chrome://browser/skin/Bookmarks-folder.png"
					}
				
					this._results.push({
						value: StringUtils.truncate(resultNode.primaryKey, truncateSize, truncateShortCut),
						comment: StringUtils.truncate(bmsvc.getItemTitle(itemIDs[0]), truncateSize, truncateShortCut),
						image: iconUri,
						style: 'suggesthint bookmarktools cl-index '
					});
				}
			}
		}
	}catch(e){
		return false;
	}
	return true;
}

/***********************************************************
class factory

This object is a member of the global-scope Components.classes.
It is keyed off of the contract ID. 

***********************************************************/

var IndexSearchFactory = { 

	createInstance: function (aOuter, aIID){   
		if (aOuter != null) throw Cr.NS_ERROR_NO_AGGREGATION;
			return (new IndexSearch()).QueryInterface(aIID);
    }
	
};

/***********************************************************
module definition (xpcom registration)
***********************************************************/

var IndexSearchModule = { 

	registerSelf: function(aCompMgr, aFileSpec, aLocation, aType) { 
		aCompMgr = aCompMgr.QueryInterface(Ci.nsIComponentRegistrar);
		aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME, CONTRACT_ID, aFileSpec, aLocation, aType);
	},
	
	unregisterSelf: function(aCompMgr, aLocation, aType) { 
		aCompMgr = aCompMgr.QueryInterface(Ci.nsIComponentRegistrar);
		aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);        
	},
	
	getClassObject: function(aCompMgr, aCID, aIID) { 
		if (!aIID.equals(Components.interfaces.nsIFactory)) throw Cr.NS_ERROR_NOT_IMPLEMENTED;
		if (aCID.equals(CLASS_ID)) return IndexSearchFactory;
		throw Cr.NS_ERROR_NO_INTERFACE;
	},
	
	canUnload: function(aCompMgr){ 
		return true;
	}
	
};

/***********************************************************
module initialization

When the application registers the component, this function
is called.
***********************************************************/

function NSGetModule(aCompMgr, aFileSpec){
	return IndexSearchModule;
};
